/*
*  Name:      defs-uesqlc.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC definitions
*  Date:      $Date: 2001/06/11 17:57:52 $
*  Revision:  $Revision: 1.49 $
*
*  Copyright (C) 1996-2001  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQLC_DEFS_UESQLC__
#define _UESQLC_DEFS_UESQLC__

#include <map>
#include <mpcl/error_exception.hh>
#include <mpcl/general-exceptions.hh>
#include <mpcl/local_date.hh>
#include <mpcl/string.hh>
#include <mpcl/warning_exception.hh>


///  Name-space for UESQL.
namespace uesql
{
  
  //
  //  E X P O R T E D   V A R I A B L E S
  //

  //
  //  These variables are used in sources generated by UESQLC.
  //

  /**
  *  SQL state parameter (values are implementation defined and using it
  *  is deprecated).
  */
  extern long int   SQLCODE;

  /**
  *  SQL extended description (target dependent). This an extended message
  *  for including target dependent description of errors.
  */
  extern mpcl::xstring   SQLMESSAGE;

  /**
  *  SQL state parameter (values in clause 22, "Status codes").
  */
  extern mpcl::xstring   SQLSTATE;

  //
  //  SQLSTATE class and subclass values.
  //    Classes has been represented as a C++ namespace, and subclasses have
  //    been represented as a constant pointer to a zero terminated string.
  //

  ///  Condition: ambiguous cursor name.
  namespace ambiguous_cursor_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS
    };
  }

  ///  Condition: cardinality violation.
  namespace cardinality_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 1000
    };
  }
  
  ///  Condition: connection exception.
  namespace connection_exception
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcCONNECTION_DOES_NOT_EXIST;
    extern const char*   pkcCONNECTION_FAILURE;
    extern const char*   pkcCONNECTION_NAME_IN_USE;
    extern const char*   pkcSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION;
    extern const char*   pkcSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION;
    extern const char*   pkcTRANSACTION_RESOLUTION_UNKNOWN;
    enum EValues
    {
      eNO_SUBCLASS = 2000                                ,
      eCONNECTION_DOES_NOT_EXIST                         ,
      eCONNECTION_FAILURE                                ,
      eCONNECTION_NAME_IN_USE                            ,
      eSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION       ,
      eSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION ,
      eTRANSACTION_RESOLUTION_UNKNOWN
    };
  }
  
  ///  Condition: data exception.
  namespace data_exception
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcCHARACTER_NOT_IN_REPERTOIRE;
    extern const char*   pkcDATETIME_FIELD_OVERFLOW;
    extern const char*   pkcDIVISION_BY_ZERO;
    extern const char*   pkcERROR_IN_ASSIGNMENT;
    extern const char*   pkcINDICATOR_OVERFLOW;
    extern const char*   pkcINTERVAL_FIELD_OVERFLOW;
    extern const char*   pkcINVALID_CHARACTER_VALUE_FOR_CAST;
    extern const char*   pkcINVALID_DATETIME_FORMAT;
    extern const char*   pkcINVALID_ESCAPE_CHARACTER;
    extern const char*   pkcINVALID_ESCAPE_SEQUENCE;
    extern const char*   pkcINVALID_PARAMETER_VALUE;
    extern const char*   pkcINVALID_TIME_ZONE_DISPLACEMENT_VALUE;
    extern const char*   pkcNULL_VALUE_NO_INDICATOR_PARAMETER;
    extern const char*   pkcNUMERIC_VALUE_OUT_OF_RANGE;
    extern const char*   pkcSTRING_DATA_LENGTH_MISMATCH;
    extern const char*   pkcSTRING_DATA_RIGHT_TRUNCATION;
    extern const char*   pkcSUBSTRING_ERROR;
    extern const char*   pkcTRIM_ERROR;
    extern const char*   pkcUNTERMINATED_C_STRING;
    enum EValues
    {
      eNO_SUBCLASS = 3000                   ,
      eCHARACTER_NOT_IN_REPERTOIRE          ,
      eDATETIME_FIELD_OVERFLOW              ,
      eDIVISION_BY_ZERO                     ,
      eERROR_IN_ASSIGNMENT                  ,
      eINDICATOR_OVERFLOW                   ,
      eINTERVAL_FIELD_OVERFLOW              ,
      eINVALID_CHARACTER_VALUE_FOR_CAST     ,
      eINVALID_DATETIME_FORMAT              ,
      eINVALID_ESCAPE_CHARACTER             ,
      eINVALID_ESCAPE_SEQUENCE              ,
      eINVALID_PARAMETER_VALUE              ,
      eINVALID_TIME_ZONE_DISPLACEMENT_VALUE ,
      eNULL_VALUE_NO_INDICATOR_PARAMETER    ,
      eNUMERIC_VALUE_OUT_OF_RANGE           ,
      eSTRING_DATA_LENGTH_MISMATCH          ,
      eSTRING_DATA_RIGHT_TRUNCATION         ,
      eSUBSTRING_ERROR                      ,
      eTRIM_ERROR                           ,
      eUNTERMINATED_C_STRING
    };
  }
  
  ///  Condition: dependent privilege descriptors still exist.
  namespace dependent_privilege_descriptors_still_exist
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 4000
    };
  }
  
  ///  Condition: dynamic SQL error.
  namespace dynamic_sql_error
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED;
    extern const char*   pkcINVALID_DESCRIPTOR_COUNT;
    extern const char*   pkcINVALID_DESCRIPTOR_INDEX;
    extern const char*   pkcPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION;
    extern const char*   pkcRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION;
    extern const char*   pkcUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC;
    extern const char*   pkcUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC;
    extern const char*   pkcUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS;
    extern const char*   pkcUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS;
    enum EValues
    {
      eNO_SUBCLASS = 5000                                 ,
      eCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED            ,
      eINVALID_DESCRIPTOR_COUNT                           ,
      eINVALID_DESCRIPTOR_INDEX                           ,
      ePREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION      ,
      eRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION           ,
      eUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC ,
      eUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC            ,
      eUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS       ,
      eUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS
    };
  }
  
  ///  Condition: feature not supported.
  namespace feature_not_supported
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcMULTIPLE_ENVIRONMENT_TRANSACTIONS;
    enum EValues
    {
      eNO_SUBCLASS = 6000                ,
      eMULTIPLE_ENVIRONMENT_TRANSACTIONS
    };
  }
  
  ///  Condition: integrity constraint violation.
  namespace integrity_constraint_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 7000
    };
  }
  
  ///  Condition: invalid authorization specification.
  namespace invalid_authorization_specification
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 8000
    };
  }
  
  ///  Condition: invalid catalog name.
  namespace invalid_catalog_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 9000
    };
  }
  
  ///  Condition: invalid character set name.
  namespace invalid_character_set_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 10000
    };
  }
  
  ///  Condition: invalid condition number.
  namespace invalid_condition_number
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 11000
    };
  }
  
  ///  Condition: invalid connection name.
  namespace invalid_connection_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 12000
    };
  }
  
  ///  Condition: invalid cursor name.
  namespace invalid_cursor_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 13000
    };
  }
  
  ///  Condition: invalid cursor state.
  namespace invalid_cursor_state
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 14000
    };
  }
  
  ///  Condition: invalid schema name.
  namespace invalid_schema_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 15000
    };
  }
  
  ///  Condition: invalid SQL descriptor name.
  namespace invalid_sql_descriptor_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 16000
    };
  }
  
  ///  Condition: invalid SQL statement name.
  namespace invalid_sql_statement_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 17000
    };
  }
  
  ///  Condition: invalid transaction state.
  namespace invalid_transaction_state
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 18000
    };
  }
  
  ///  Condition: invalid transaction termination.
  namespace invalid_transaction_termination
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 19000
    };
  }
  
  ///  Condition: no data.
  namespace no_data
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 20000
    };
  }
  
  ///  Condition: remote database access.
  namespace remote_database_access
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 21000
    };
  }
  
  ///  Condition: successful completion.
  namespace successful_completion
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 22000
    };
  }
  
  ///  Condition: syntax error or access rule violation.
  namespace syntax_error_or_access_rule_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 23000
    };
  }
  
  ///  Condition: syntax error or access rule violation in direct statement.
  namespace syntax_error_or_access_rule_violation_in_direct_statement
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 24000
    };
  }
  
  ///  Condition: syntax error or access rule violation in dynamic statement.
  namespace syntax_error_or_access_rule_violation_in_dynamic_statement
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 25000
    };
  }
  
  ///  Condition: transaction rollback.
  namespace transaction_rollback
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcINTEGRITY_CONSTRAINT_VIOLATION;
    extern const char*   pkcSERIALIZATION_FAILURE;
    extern const char*   pkcSTATEMENT_COMPLETION_UNKNOWN;
    enum EValues
    {
      eNO_SUBCLASS = 26000            ,
      eINTEGRITY_CONSTRAINT_VIOLATION ,
      eSERIALIZATION_FAILURE          ,
      eSTATEMENT_COMPLETION_UNKNOWN
    };
  }
  
  ///  Condition: triggered data change violation.
  namespace triggered_data_change_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 27000
    };
  }
  
  ///  Condition: warning.
  namespace warning
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcCURSOR_OPERATION_CONFLICT;
    extern const char*   pkcDISCONNECT_ERROR;
    extern const char*   pkcIMPLICIT_ZERO_BIT_PADDING;
    extern const char*   pkcINSUFFICIENT_ITEM_DESCRIPTOR_AREAS;
    extern const char*   pkcNULL_VALUE_ELIMINATED_IN_SET_FUNCTION;
    extern const char*   pkcPRIVILEGE_NOT_GRANTED;
    extern const char*   pkcPRIVILEGE_NOT_REVOKED;
    extern const char*   pkcQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA;
    extern const char*   pkcSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA;
    extern const char*   pkcSTRING_DATA_RIGHT_TRUNCATION;
    enum EValues
    {
      eNO_SUBCLASS = 28000                              ,
      eCURSOR_OPERATION_CONFLICT                        ,
      eDISCONNECT_ERROR                                 ,
      eIMPLICIT_ZERO_BIT_PADDING                        ,
      eINSUFFICIENT_ITEM_DESCRIPTOR_AREAS               ,
      eNULL_VALUE_ELIMINATED_IN_SET_FUNCTION            ,
      ePRIVILEGE_NOT_GRANTED                            ,
      ePRIVILEGE_NOT_REVOKED                            ,
      eQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA ,
      eSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA ,
      eSTRING_DATA_RIGHT_TRUNCATION
    };
  }
  
  ///  Condition: with check option violation.
  namespace with_check_option_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 29000
    };
  }

  ///  Base SQL data type. It is used as base class for embedded variables.
  class TSqlDataType
  {

    protected:

      ///  If true, then data retrieved from SQL server, is null.
      bool   gIsNull;


    public:
      
      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TSqlDataType (void) :
        gIsNull (false)   {}

      ///  Builds a new instance from another instance.
      TSqlDataType (const TSqlDataType& rktSQL_DATA_TYPE) :
        gIsNull (rktSQL_DATA_TYPE.gIsNull)                {}

      ///  Destroys the instance.
      virtual ~TSqlDataType (void) {}
      
      /**
      *  Returns a buffer where you can assign a value of, at most,
      *  \a kzSIZE octets.
      *  @param kzSIZE Octet count.
      *  @return A pointer to the buffer.
      */
      virtual void* getBuffer (const std::size_t kzSIZE) = 0;

      /**
      *  Returns a buffer where you can assign a value of, at most,
      *  \a kzSIZE octets.
      *  @param kzSIZE Octet count.
      *  @return A pointer to the buffer.
      */
      virtual void updateFromBuffer (void) = 0;

      /**
      *  Assigns value \a pkcVALUE.
      *  @param pkcVALUE Source value.
      */
      virtual void set (const char* pkcVALUE) = 0;
      
      /**
      *  Assigns value \a rkyVALUE.
      *  @param rkyVALUE Source value.
      */
      virtual void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      virtual mpcl::xstring get (void) const = 0;

      /**
      *  Returns true if the last data retrieved has been a NULL value.
      *  @return True if the last data retrieved has been a NULL value.
      */
      bool isNull (void) const
      {
        return gIsNull;
      }

  };  // class TSqlDataType

  ///  SQL date handler.
  class TSqlDate            :
    public mpcl::TLocalDate ,
    public TSqlDataType
  {

    private:
      
      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;


    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance from another instance.
      TSqlDate (const TSqlDate& rktSQL_DATE) :
        mpcl::TLocalDate (rktSQL_DATE)       ,
        TSqlDataType     (rktSQL_DATE)       ,
        pcBuffer         (NULL)              {}

      TSqlDate (const EDateType keDATE_TYPE = eNow) :
        mpcl::TLocalDate (keDATE_TYPE)              ,
        TSqlDataType     ()                         ,
        pcBuffer         (NULL)                     {}

      TSqlDate (const int kiYEAR, const int kiMONTH, const int kiDAY) :
        mpcl::TLocalDate (kiYEAR, kiMONTH, kiDAY)                     ,
        TSqlDataType     ()                                           ,
        pcBuffer         (NULL)                                       {}

      TSqlDate (const char* pkcDATE);

      TSqlDate (const std::string& rkyDATE);

      ///  Destroys the instance.
      virtual ~TSqlDate (void);

      /**
      *  Reserves  a  buffer \a kzSIZE  octets  length,  to  let somebody store
      *  a date as a zero-terminated string of chars.
      *  @param kzSIZE Size of the buffer.
      *  @return The buffer.
      */
      void* getBuffer (const std::size_t kzSIZE);

      void set (const char* pkcVALUE);

      void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }

      void updateFromBuffer (void);


    public:

      //
      //  S E L E C T O R S
      //

      mpcl::xstring get (void) const
      {
        return TLocalDate::get ("'%Y-%m-%d'");
      }

      /**
      *  Returns date in SQL format.
      *  @param pkcDATE_FORMAT Date format in strftime-like format. It defaults
      *                        to ANSI SQL date  format.  A  valid date  can be
      *                        '1970-12-31'.
      *  @return Date in readable format.
      *  @warning pkcDATE_FORMAT must include the enclosing quotes (').
      */
      mpcl::xstring get (const char* pkcDATE_FORMAT) const
      {
        return TLocalDate::get (pkcDATE_FORMAT);
      }
      
  };  // class TSqlDate

  ///  SQL type INTEGER.
  class TSqlInteger : public TSqlDataType
  {

    private:

      ///  Integer value.
      long int   liValue;

      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;


    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TSqlInteger (void)    :
        TSqlDataType ()     ,
        liValue      (0L)   ,
        pcBuffer     (NULL) {}

      ///  Builds a new instance from another instance.
      TSqlInteger (const TSqlInteger& rktSQL_INTEGER) :
        TSqlDataType (rktSQL_INTEGER)                 ,
        liValue      (rktSQL_INTEGER.liValue)         ,
        pcBuffer     (NULL)                           {}

      TSqlInteger (const long int kliVALUE) :
        TSqlDataType ()                     ,
        liValue      (kliVALUE)             ,
        pcBuffer     (NULL)                 {}

      TSqlInteger (const char* pkcSQL_INTEGER)    :
        TSqlDataType ()                           ,
        liValue      (std::atol (pkcSQL_INTEGER)) ,
        pcBuffer     (NULL)                       {}

      TSqlInteger (const std::string& rkySQL_INTEGER)     :
        TSqlDataType ()                                   ,
        liValue      (std::atol (rkySQL_INTEGER.c_str())) ,
        pcBuffer     (NULL)                               {}

      ///  Destroys the instance.
      virtual ~TSqlInteger (void);
      
      void* getBuffer (const std::size_t kzSIZE);

      void updateFromBuffer (void);

      void set (const char* pkcVALUE);
      
      void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::xstring get (void) const
      {
        return mpcl::Format ("%ld", liValue);
      }

      ///  Returns the long integer value.
      operator long int (void) const
      {
        return liValue;
      }

  };  // class TSqlInteger

  ///  SQL type REAL.
  class TSqlReal : public TSqlDataType
  {

    private:

      ///  Real value.
      double   dValue;

      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;


    public:

      //
      //  C O N S T R U C T O R S
      //

      TSqlReal (void)       :
        TSqlDataType ()     ,
        dValue       (0L)   ,
        pcBuffer     (NULL) {}

      ///  Builds a new instance from another instance.
      TSqlReal (const TSqlReal& rktSQL_REAL) :
        TSqlDataType (rktSQL_REAL)           ,
        dValue       (0L)                    ,
        pcBuffer     (NULL)                  {}

      TSqlReal (const double kdVALUE) :
        TSqlDataType ()               ,
        dValue       (kdVALUE)        ,
        pcBuffer     (NULL)           {}

      TSqlReal (const char* pkcSQL_REAL)       :
        TSqlDataType ()                        ,
        dValue       (std::atof (pkcSQL_REAL)) ,
        pcBuffer     (NULL)                    {}

      TSqlReal (const std::string& rkySQL_REAL)        :
        TSqlDataType ()                                ,
        dValue       (std::atof (rkySQL_REAL.c_str())) ,
        pcBuffer     (NULL)                            {}

      virtual ~TSqlReal (void);
      
      void* getBuffer (const std::size_t kzSIZE);

      void updateFromBuffer (void);

      void set (const char* pkcVALUE);
      
      void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::xstring get (void) const
      {
        return mpcl::Format ("%f", dValue);
      }

      /**
      *  Returns a string.
      *  @param pkcFORMAT Format string  for  converting from a double value to
      *                   string.
      *  @return The result of converting this instance to string.
      *  @see std::printf(), mpcl::Format().
      */
      mpcl::xstring get (const char* pkcFORMAT) const
      {
        return mpcl::Format (pkcFORMAT, dValue);
      }

      ///  Returns the SQL real value.
      operator long double (void) const
      {
        return dValue;
      }

  };  // class TSqlReal

  ///  SQL type string.
  class TSqlString       :
    public TSqlDataType  ,
    public mpcl::xstring
  {

    private:
      
      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;
      

    public:

      ///  Import this type.
      typedef mpcl::xstring::size_type size_type;


    public:
      
      //
      //  C O N S T R U C T O R S
      //
      
      explicit TSqlString (void) :
        TSqlDataType  ()         ,
        mpcl::xstring ()         ,
        pcBuffer      (NULL)     {}

      ///  Builds a new instance from another instance.
      TSqlString (const TSqlString& rkySQL_STRING) :
        TSqlDataType  (rkySQL_STRING)              ,
        mpcl::xstring (rkySQL_STRING)              ,
        pcBuffer      (NULL)                       {}
      
      TSqlString (const mpcl::xstring& rkySOURCE) :
        TSqlDataType  ()                          ,
        mpcl::xstring (rkySOURCE)                 ,
        pcBuffer      (NULL)                      {}
      
      TSqlString (const TSqlString& rkySQL_STRING  ,
                  size_type         zOFFSET        ,
                  size_type         zLENGTH = npos )    :
        TSqlDataType  (rkySQL_STRING)                   ,
        mpcl::xstring (rkySQL_STRING, zOFFSET, zLENGTH) ,
        pcBuffer      (NULL)                            {}

      /**
      *  Extends the parent constructor with checks for NULL string.
      *  @param pkcVALUE Source string.
      *  @param zLENGTH  Source string length.
      *  @exception TExceptionEL_Constraint If source string is NULL.
      */
      TSqlString (const char* pkcVALUE, size_type zLENGTH) :
        TSqlDataType  ()                                   ,
        mpcl::xstring ()                                   ,
        pcBuffer      (NULL)
      {
        if ( !pkcVALUE )
        {
          throw mpcl::TExceptionEL_Constraint ("null string", __FILE__, __LINE__);
        }
        assign (pkcVALUE, zLENGTH);
      }
      
      /**
      *  Extends the parent constructor with checks for NULL string.
      *  @param pkcVALUE Source string.
      *  @exception TExceptionEL_Constraint If source string is NULL.
      */
      TSqlString (const char* pkcVALUE) :
        TSqlDataType  ()                ,
        mpcl::xstring ()                ,
        pcBuffer      (NULL)
      {
        if ( !pkcVALUE )
        {
          throw mpcl::TExceptionEL_Constraint ("null string", __FILE__, __LINE__);
        }
        assign (pkcVALUE);
      }
      
      TSqlString (size_type zTIMES, char cSOURCE) :
        TSqlDataType  ()                          ,
        mpcl::xstring (zTIMES, cSOURCE)           ,
        pcBuffer      (NULL)                      {}
      
      template<typename InputIterator>
      TSqlString(InputIterator tBEGIN_ITER, InputIterator tEND_ITER) :
        TSqlDataType  ()                                             ,
        mpcl::xstring (tBEGIN_ITER, tEND_ITER)                       ,
        pcBuffer      (NULL)                                         {}

      ~TSqlString (void);
      
      TSqlString& operator = (const TSqlString& rkySQL_STRING)
      {
        mpcl::xstring::operator = (rkySQL_STRING);
        return *this;
      }

      TSqlString& operator = (const mpcl::xstring& rkySOURCE)
      {
        mpcl::xstring::operator = (rkySOURCE);
        return *this;
      }

      TSqlString& operator = (const char* pkcVALUE)
      {
        if ( !pkcVALUE )
        {
          throw mpcl::TExceptionEL_Constraint ("null string", __FILE__, __LINE__);
        }
        mpcl::xstring::operator = (pkcVALUE);
        return *this;
      }

      TSqlString& operator = (char cSOURCE)
      {
        mpcl::xstring::operator = (cSOURCE);
        return *this;
      }

      void* getBuffer (const std::size_t kzSIZE);

      void set (const char* pkcVALUE)
      {
        gIsNull = ( !pkcVALUE );
        if ( gIsNull )
        {
          erase();
        }
        else
        {
          assign (pkcVALUE);
        }
      }

      void set (const std::string& rkyVALUE)
      {
        assign (rkyVALUE);
      }

      void updateFromBuffer (void);


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::xstring get (void) const
      {
        return *this;
      }

  };  // class TSqlString

  ///  Handler for SQLSTATE status parameter.
  class TSqlstateDescriptor
  {

    private:
      
      /**
      *  Map with SQLSTATE values (strings) to integer translations.
      *  So the integer values are suitable for a \b switch statement.
      */
      static map<mpcl::xstring, int>   _tValueToIntMap;


    protected:
      
      //
      //  L O C A L   F U N C T I O N S
      //

      /**
      *  Adds a new pair (SQLSTATE, iVALUE) to the map.
      *  @param pkcSQLSTATE SQL state description.
      *  @param iVALUE      SQL state unique integer value.
      */
      static void _addCondition ( const char* pkcSQLSTATE ,
                                  int         iVALUE      );

      /**
      *  Initializes the SQLSTATE to integer map.
      */
      static void _initialize (void);
      
      
    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance and initializes the map.
      TSqlstateDescriptor (void)
      {
        _initialize();
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns true if the query is empty.
      *  @return True if query is empty.
      */
      bool queryIsEmpty (void) const
      {
        return ( getUniqueValue() == no_data::eNO_SUBCLASS );
      }

      /**
      *  Returns true if the query has only a single row.
      *  @return True if query has a single row.
      */
      bool queryHasSingleRow (void) const
      {
        //
        //  Check if query has generated only *one* row.
        //
        return ( ( getUniqueValue() != cardinality_violation::eNO_SUBCLASS ) &&
                 ( getUniqueValue() != no_data::eNO_SUBCLASS )                );
      }

      /**
      *  Finds the integer value for the current SQLSTATE.
      *  @return The integer value for the current SQLSTATE.
      */
      int getUniqueValue (void) const
      {
        int                                       iUniqueValue = -1;
        map<mpcl::xstring, int>::const_iterator   I            = _tValueToIntMap.find (SQLSTATE);

        if ( I != _tValueToIntMap.end() )
        {
          iUniqueValue = I->second;
        }        
        return iUniqueValue;
      }

      /**
      *  Finds the integer value for the SQLSTATE in \a pkcSQLSTATE.
      *  @return The integer value for the SQLSTATE in \a pkcSQLSTATE.
      */
      int getUniqueValue (const char* pkcSQLSTATE) const
      {
        return _tValueToIntMap [pkcSQLSTATE];
      }

      /**
      *  Get SQLSTATE class.
      *  @return Class description.
      */
      const char* getClass (void) const
      {
        return getClass (SQLSTATE.c_str());
      }
      
      /**
      *  Get SQLSTATE class for \a pkcSQLSTATE.
      *  @return Class description.
      */
      const char* getClass (const char* pkcSQLSTATE) const;
      
      /**
      *  Gets  the  complete  description for the SQLSTATE as  in  the  source
      *  parameter \a pkcSQLSTATE.
      *  @return Complete description of the SQLSTATE in .
      *  @return Class description.
      */
      mpcl::xstring getDescription (const char* pkcSQLSTATE) const;
      
      /**
      *  Get SQLSTATE subclass.
      *  @return Subclass description.
      */
      const char* getSubclass (void) const
      {
        return getSubclass (SQLSTATE.c_str());
      }
      
      /**
      *  Get SQLSTATE subclass for \a pkcSQLSTATE.
      *  @return Subclass description.
      */
      const char* getSubclass (const char* pkcSQLSTATE) const;

  };  // class TSqlstateDescriptor


  ///  Exception: SQL exception class for NOT FOUND condition.
  class TExceptionUESQL_NotFound        :
    public virtual mpcl::TBaseException ,
    public mpcl::TWarningException      ,
    private TSqlstateDescriptor
  {

    public:

      /**
      *  Builds a new instance.
      *  @param pkcFILE_NAME  Filename where the exception has occurred.
      *  @param kiLINE_NUMBER Line number in the file where the exception has
      *                       occurred.
      */
      TExceptionUESQL_NotFound ( const char* pkcFILE_NAME  ,
                                 const int   kiLINE_NUMBER );


    public:

      //
      //  S E L E C T O R S
      //

      const std::string what (void) const;

  };  // class TExceptionUESQL_NotFound


  ///  Exception: SQL exception class for SQLERROR condition.
  class TExceptionUESQL_Sqlerror        :
    public virtual mpcl::TBaseException ,
    public mpcl::TErrorException        ,
    private TSqlstateDescriptor
  {

    public:

      /**
      *  Builds a new instance.
      *  @param pkcFILE_NAME  Filename where the exception has occurred.
      *  @param kiLINE_NUMBER Line number in the file where the exception has
      *                       occurred.
      */
      TExceptionUESQL_Sqlerror ( const char* pkcFILE_NAME  ,
                                 const int   kiLINE_NUMBER );


    public:

      //
      //  S E L E C T O R S
      //

      const std::string what (void) const;

  };  // class TExceptionUESQL_Sqlerror


  //
  //  E X P O R T E D   F U N C T I O N S
  //

  //
  //  Simple n-adic functions for embedded function calls.
  //

  /**
  *  Composes a string as the concatenation of a quote, \a rkyHOST_NAME,
  *  a solidus, a \a rkySCHEMA_NAME and then a quote.
  */
  TSqlString ComposeQuotedSchemaLocation ( const std::string& rkyHOST_NAME   ,
                                           const std::string& rkySCHEMA_NAME );

  /**
  *  Composes a string  as  the  concatenation  of  a quote, \a rkyUSER_NAME, a
  *  solidus, a \a rkyUSER_PASSWORD and then a quote.
  */
  TSqlString ComposeQuotedUserLogin ( const std::string& rkyUSER_NAME     ,
                                      const std::string& rkyUSER_PASSWORD );

  /**
  *  Composes a string as the  concatenation of \a rkyHOST_NAME, a solidus, and
  *  then \a rkySCHEMA_NAME.
  */
  TSqlString ComposeSchemaLocation ( const std::string& rkyHOST_NAME   ,
                                     const std::string& rkySCHEMA_NAME );

  /**
  *  Composes a string as the  concatenation of \a rkyUSER_NAME, a solidus, and
  *  then \a rkyUSER_PASSWORD.
  */
  TSqlString ComposeUserLogin ( const std::string& rkyUSER_NAME     ,
                                const std::string& rkyUSER_PASSWORD );

  /**
  *  Quotes the source string \a rkySTRING with single quotes. This function is
  *  used in terminal <embedded n-adic function call>.
  *  @param rkySTRING Source string.
  *  @return The source string between single quotes.
  */
  TSqlString Quote (const std::string& rkySTRING);
  
  /**
  *  Quotes the source string \a pkcSTRING with single quotes.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param pkcSTRING Source string.
  *  @return The source string between single quotes.
  */
  TSqlString Quote (const char* pkcSTRING);

  /**
  *  Converts \a gVALUE to a string.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param gVALUE Source boolean value.
  *  @return A string with "0" (if value is false) or "1" (if value is true).
  */
  TSqlString ToString (bool gVALUE);
  
  /**
  *  Converts \a ldNUMBER to a string.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param ldNUMBER Source number.
  *  @return A string with the value of \a ldNUMBER.
  */
  TSqlString ToString (long double ldNUMBER);
  
  /**
  *  Converts \a liNUMBER to a string.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param liNUMBER Source number.
  *  @return A string with the value of \a liNUMBER.
  */
  TSqlString ToString (long int liNUMBER);
  
  /**
  *  Converts \a luiNUMBER to a string.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param luiNUMBER Source number.
  *  @return A string with the value of \a luiNUMBER.
  */
  TSqlString ToString (long unsigned int luiNUMBER);
  
}  // namespace uesql


#endif  // not _UESQLC_DEFS_UESQLC__
