/*
*  Name:         destroy-iterators-db.uesql.cc
*  Author:       Francisco Vides Fernandez
*  Summary:      Destroy iterators test database
*  Date:         $$
*  Revision:     $$
*
*  Copyright (C) 2001  Francisco Vides Fernandez <pax@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/general-exceptions.hh>
#include <mpcl/log.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>

using uesql::TExceptionUESQL_Sqlerror;


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;


//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::xstring _GetErrorMessage (void)
{

  using mpcl::xstring;

  return ( xstring (_tSqlstateDescriptor.getClass())    +
           ": "                                         +
           xstring (_tSqlstateDescriptor.getSubclass()) +
           " (" + uesql::SQLMESSAGE + ")"               );

}  // _GetErrorMessage()


static void _DropTestSchema ( const mpcl::xstring& rkyDBMS_SERVER_NAME ,
                              const mpcl::xstring& rkyUSER_NAME        ,
                              const mpcl::xstring& rkyUSER_PASSWORD    )
{

  using mpcl::xstring;
  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::Quote;

  xstring   yDbmsSchemaLocation;
  xstring   yDbmsTestSchemaLocation;
  xstring   yDbmsUserLogin;

  yDbmsSchemaLocation     = ComposeSchemaLocation (rkyDBMS_SERVER_NAME, rkyUSER_NAME);
  yDbmsTestSchemaLocation = ComposeSchemaLocation (rkyDBMS_SERVER_NAME, "TEST_ITERATORS_SCHEMA");
  yDbmsUserLogin          = ComposeUserLogin (rkyUSER_NAME, rkyUSER_PASSWORD);
  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION      = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN           = %s", yDbmsUserLogin.c_str());
  TEST_WRITE_INFO ("DBMS-TEST-SCHEMA-LOCATION = %s", yDbmsTestSchemaLocation.c_str());
  extern "UESQL::default"
  {
    --
    --  Connects to default schema.
    --
    connect to :Quote (yDbmsSchemaLocation)
      user :Quote (yDbmsUserLogin);
    --
    --  Removes schema test_iterators_schema.
    --
    drop schema test_iterators_schema cascade;
  }

}   // _DropTestSchema()


int main (int argc, const char** argv)
{

  mpcl::TConfigProcessor   tConfigProcessor (argc, argv);

  TEST_INIT ("Test for Cursor inside C++ Iterators");
  TEST_START_WATCH;
  try
  {
    tConfigProcessor.addOption ("dbms-server-name",   "o", "dbms");
    tConfigProcessor.addOption ("dbms-user-name",     "u", "test");
    tConfigProcessor.addOption ("dbms-user-password", "p", "test");
    tConfigProcessor["dbms-server-name"].setUsageString ("Server where database resides");
    tConfigProcessor["dbms-user-name"].setUsageString ("");
    tConfigProcessor["dbms-user-password"].setUsageString ("");
    tConfigProcessor.processOptions();
    
    if ( !TEST_IN_SILENT_MODE )
    {
      mpcl::tLog.also (cerr);
    }
    extern "UESQL::default" 
    {
      --
      --  Whenever any SQLERROR condition throw an exception
      --
      whenever sqlerror throw TExceptionUESQL_Sqlerror;
      
    }
    _DropTestSchema ( tConfigProcessor ["dbms-server-name"].getValue()   ,
                      tConfigProcessor ["dbms-user-name"].getValue()     ,
                      tConfigProcessor ["dbms-user-password"].getValue() );
  }
  catch (const uesql::TExceptionUESQL_Sqlerror& rktEXCEPTION)
  {
    cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (...)
  {
    cerr << "Unhandled exception" << endl;
    TEST_FAIL_WITH_CODE (3);
  }
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
